/*
 *  Abstraction layer for MIXER
 *  Copyright (c) by Jaroslav Kysela <perex@jcu.cz>
 */
 
#define SND_MIXER_DEVS	16		/* max mixer devices per card */

#define SND_MIXER_CAP_EXCL_INPUT      0x00000001      /* only one recording source at a time */

#define SND_MIXER_OSSDEVS	17
#define SND_MIXER_VOLUME	0
#define SND_MIXER_BASS		1
#define SND_MIXER_TREBLE	2
#define SND_MIXER_SYNTH		3
#define SND_MIXER_PCM		4
#define SND_MIXER_SPEAKER	5
#define SND_MIXER_LINE		6
#define SND_MIXER_MIC		7
#define SND_MIXER_CD		8
#define SND_MIXER_IMIX		9		/* recording monitor */
#define SND_MIXER_ALTPCM	10
#define SND_MIXER_RECLEV	11		/* recording level */
#define SND_MIXER_IGAIN		12		/* input gain */
#define SND_MIXER_OGAIN		13		/* output gain */
#define SND_MIXER_LINE1		14
#define SND_MIXER_LINE2		15
#define SND_MIXER_LINE3		16
#define SND_MIXER_UNKNOWN	(32+1)

struct snd_oss_mixer_info {
  char id[ 16 ];
  char name[ 32 ];
};
    
#define SND_MIXER_SET_RECSRC	_IOWR( 'M', 255, int )
#define SND_MIXER_RECSRC	_IOR ( 'M', 255, int )
#define SND_MIXER_DEVMASK	_IOR ( 'M', 254, int )
#define SND_MIXER_RECMASK	_IOR ( 'M', 253, int )
#define SND_MIXER_CAPS		_IOR ( 'M', 252, int )
#define SND_MIXER_STEREODEVS	_IOR ( 'M', 251, int )
#define SND_MIXER_OSS_INFO   	_IOR ( 'M', 101, struct snd_oss_mixer_info )

#define SND_MIXER_PRI_MASTER		0x00000100
#define SND_MIXER_PRI_SYNTHESIZER	0x00000200
#define SND_MIXER_PRI_SYNTHESIZER1	0x00000300
#define SND_MIXER_PRI_FM		0x00000400
#define SND_MIXER_PRI_EFFECT		0x00000500
#define SND_MIXER_PRI_PCM		0x00000600
#define SND_MIXER_PRI_PCM1		0x00000700
#define SND_MIXER_PRI_LINE		0x00000800
#define SND_MIXER_PRI_MIC		0x00000900
#define SND_MIXER_PRI_CD		0x00000a00
#define SND_MIXER_PRI_GAIN		0x00000b00
#define SND_MIXER_PRI_IGAIN		0x00000c00
#define SND_MIXER_PRI_OGAIN		0x00000d00
#define SND_MIXER_PRI_LOOPBACK		0x00000e00
#define SND_MIXER_PRI_SPEAKER		0x00000f00
#define SND_MIXER_PRI_AUXA		0xf0000000
#define SND_MIXER_PRI_AUXB		0xf0000100
#define SND_MIXER_PRI_AUXC		0xf0000200

#define SND_MIX_MUTE_LEFT	1
#define SND_MIX_MUTE_RIGHT	2
#define SND_MIX_MUTE		(SND_MIX_MUTE_LEFT|SND_MIX_MUTE_RIGHT)

struct SND_STRU_MIXER_CHANNEL;
struct SND_STRU_MIXER_FILE;
struct SND_STRU_MIXER;

typedef struct snd_stru_mixer_channel snd_kmixer_channel_t;
typedef struct snd_stru_mixer_file snd_kmixer_file_t;
typedef struct snd_stru_mixer snd_kmixer_t;

struct snd_stru_mixer_channel_hw {
  unsigned int priority;	/* this is sorting key - highter value = end */
  char name[ 16 ];		/* device name */
  unsigned short ossdev;	/* assigned OSS device number */
  unsigned short mute: 1,	/* mute is supported */
                 stereo: 1,	/* stereo is supported */
                 record: 1;	/* record is supported */
  unsigned int max;		/* max left & right value (from 0) */
  int min_dB, max_dB, step_dB;	/* min_dB, max_dB, step_dB */
  unsigned int private_value;	/* can be used by low-level driver */

  /* input = 0 to 100 (user volume), output = 0 to max (low-level value) */
  /* NULL -> linear step */
  unsigned int (*compute_volume)( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel, unsigned int volume );
  /* input = dB value from application, output = 0..100 (linear volume) */
  unsigned char (*compute_linear)( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel, int dB );
  /* input = 0 to 100 (user volume), output = dB value */
  int (*compute_dB)( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel, unsigned char avolume );
  /* --- */
  void (*set_record_source)( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel, int enable );
  void (*set_mute)( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel, unsigned int mute );
  void (*set_volume_level)( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel, unsigned int left, unsigned int right );
};

struct snd_stru_mixer_channel {
  unsigned short channel;	/* channel index */
  unsigned short record: 1;	/* recording is enabled */
 
  unsigned char umute;		/* user mute */
  unsigned char kmute;		/* kernel mute */

  unsigned char mute;		/* real mute flags */
  unsigned char pad;		/* reserved */

  unsigned char aleft;		/* application - 0 to 100 */
  unsigned char aright;		/* application - 0 to 100 */
  unsigned int uleft;		/* user - 0 to max */
  unsigned int uright;		/* user - 0 to max */
  unsigned int left;		/* real - 0 to max */
  unsigned int right;		/* real - 0 to max */

  unsigned int private_data;
  
  struct snd_stru_mixer_channel_hw hw; /* readonly variables */ 
};

struct snd_stru_mixer_file {
  snd_kmixer_t *mixer;
  volatile unsigned int changes;
  SLEEP_DEFINE( change );
  struct snd_stru_mixer_file *next;
};

struct snd_stru_mixer_hw {
  unsigned int caps;
  int (*set_special)( snd_kmixer_t *mixer, struct snd_mixer_special *special );
  int (*get_special)( snd_kmixer_t *mixer, struct snd_mixer_special *special );
};

struct snd_stru_mixer {
  snd_card_t *card;

  char id[ 32 ];
  unsigned char info[ 64 ];

  unsigned short port;

  unsigned int channels_count;		/* channels count */
  snd_kmixer_channel_t *channels[ SND_MIXER_DEVS ];

  void *private_data;
  void (*private_free)( void *private_data );
  unsigned int private_value;

  struct snd_stru_mixer_hw hw; 		/* readonly variables */

  snd_kmixer_file_t *ffile;		/* first file */
  MUTEX_DEFINE( ffile );
};

extern void snd_mixer_set_kernel_mute( snd_kmixer_t *mixer, unsigned int priority, unsigned short mute );

extern snd_kmixer_t *snd_mixer_new( snd_card_t *card, char *id, char *info, unsigned short port );
extern int snd_mixer_free( snd_kmixer_t *mixer );

extern snd_kmixer_channel_t *snd_mixer_new_channel( snd_kmixer_t *mixer, struct snd_stru_mixer_channel_hw *hw );
extern void snd_mixer_reorder_channel( snd_kmixer_t *mixer, snd_kmixer_channel_t *channel );

extern int snd_mixer_register( snd_kmixer_t *mixer, int device );
extern int snd_mixer_unregister( snd_kmixer_t *mixer );

extern int snd_mixer_channel_init( snd_kmixer_t *mixer, unsigned int priority, unsigned char left, unsigned char right, unsigned int flags );
